import numpy as np
import scipy.optimize as op

# Zheng et al 2018 JCP 148:123329
# Function for solving distance from FRET efficiency using SAW-nu 

# input
# E: FRET efficiency; N: number of peptide bonds
# r0: R0 of FRET dye; prefactor: 0.55 nm.  We find 0.55 nm good for a well-mixed IDP sequence.
# r: distance vector for integration
# output
# d2saw: squared distance from SAW-nu
# nusaw: polymer scaling exponent from SAW-nu
def d2_saw(fe,N,r0,r,prefactor):
	rc=(r[1:]+r[:-1])/2.
	dr=r[1:]-r[:-1]
	res=fe.copy()
	new_nu=fe.copy()
	def func_fe(b,fe,N,prefactor):
		nu=np.log(np.sqrt(b)/prefactor)/np.log(N)
		prtmp=pr_saw(rc,b,nu)
		return np.sum(1./(1+(rc/r0)**6)*prtmp*dr)-fe
	for idx in range(len(fe)):
		## (prefactor*N**0.5)**2 is the initial guess of squared distance
		res[idx]=op.fsolve(func_fe,x0=[(prefactor*N**0.5)**2],args=(fe[idx],N,prefactor))
	new_nu=np.log(np.sqrt(res)/prefactor)/np.log(N)
	return res,new_nu
# P(r) of SAW-nu
def pr_saw(r,b,nu):
	g=(1.1615-1)/nu
	d=1./(1-nu)
	def tmp_saw(a,x):
		xc=(x[1:]+x[:-1])/2.
		dx=x[1:]-x[:-1]
		tmp=a[0]*xc**g*np.exp(-a[1]*xc**d)
		return (np.sum(tmp*4*np.pi*xc**2*dx)-1,np.sum(xc**2*tmp*4*np.pi*xc**2*dx)-1)
	x=r/np.sqrt(b)
	a=op.fsolve(tmp_saw,x0=[1.0,1.0],args=(x))
	prtmp=a[0]*x**g*np.exp(-a[1]*x**d)/(b)**1.5*4*np.pi*r**2
	return prtmp
# <R^2>/<Rg^2> as a function of the polymer scaling exponent
def ratio(nu):
	r=1.1615
	return 2*(r+2*nu)*(r+2*nu+1)/r/(r+1)

# Gaussian
# P(r)
def pr_gaussian(r,b):
	return 4*np.pi*r**2/(2./3*np.pi*b)**1.5*np.exp(-3*r**2/2/b)
# squared distance from FRET efficiency
def d2_gaussian(fe,r0,r):
	rc=(r[1:]+r[:-1])/2.
	dr=r[1:]-r[:-1]
	def func_fe(b,fe):
		prtmp=pr_gaussian(rc,b)
		return np.sum(1./(1+(rc/r0)**6)*prtmp*dr)-fe
	res=fe.copy()
	for idx in range(len(fe)):
		res[idx]=op.fsolve(func_fe,x0=[0.5],args=(fe[idx]))
	return res
